import pandas as pd
from sklearn.preprocessing import LabelEncoder

class DataLoader:
    def __init__(self, dataset_path):
        self.dataset_path = dataset_path

    def load_data(self):
        return pd.read_csv(self.dataset_path)


class LabelProcessor:
    def __init__(self, dataset, label_column):
        self.dataset = dataset
        self.label_column = label_column
        self.label_encoder = LabelEncoder()

    def encode_labels(self):
        self.dataset[self.label_column] = self.label_encoder.fit_transform(self.dataset[self.label_column])
        return self.dataset

    def get_class_names(self):
        return self.label_encoder.classes_

    def save_class_names(self, class_names, output_path):
        with open(output_path, 'w', encoding='utf-8') as file:
            for class_name in class_names:
                file.write(f"{class_name}\n")


def process_dataset(dataset_path, label_column, output_path):
    # Load data
    data_loader = DataLoader(dataset_path)
    dataset = data_loader.load_data()

    # Process labels
    label_processor = LabelProcessor(dataset, label_column)
    dataset = label_processor.encode_labels()
    class_names = label_processor.get_class_names()

    # Save classification names
    label_processor.save_class_names(class_names, output_path)

    return dataset


def main():
    positive_dataset_path = '../dataset/filtered/filtered_dataset_positive.csv'
    negative_dataset_path = '../dataset/filtered/filtered_dataset_negative.csv'
    positive_output_path = './class_names_positive.txt'
    negative_output_path = './class_names_negative.txt'

    # Process positive dataset
    process_dataset(positive_dataset_path, 'rubrics', positive_output_path)

    # Process negative dataset
    process_dataset(negative_dataset_path, 'rubrics', negative_output_path)


if __name__ == "__main__":
    main()
