from typing import Annotated
from fastapi import APIRouter, Depends, HTTPException
from typing import List

from pydantic import ValidationError

from enums import TypeMood, TypeModel
from genetic_algorithm.genetic_algorithm import genetic_algorithm, load_graph_from_request
from repository import QuestionRepository
from schemas import SQuestionAdd, SQuestion, SQuestionId, Flight, TripRequest

router = APIRouter(
    prefix="/class",
    tags=["Class"],
)


@router.post("/get_flight/")
async def get_flight(request: TripRequest):
    graphTo, graphBack, start_point, end_point, flightsDataTo, flightsDataBack = load_graph_from_request(request)
    if request.returnDate:
        resultTo = genetic_algorithm(start=start_point, end=end_point, graph=graphTo, flights_data=flightsDataTo, type = "to")
        resulFrom = genetic_algorithm(end=start_point, start=end_point, graph=graphBack, flights_data=flightsDataBack, type = "back")
        if not resultTo or not resulFrom:
            raise HTTPException(status_code=404, detail="No valid paths found")
        return resultTo, resulFrom
    if not request.returnDate:
        resultTo = genetic_algorithm(start=start_point, end=end_point, graph=graphTo, flights_data=flightsDataTo,
                                     type="to")
        resultTo.append({
            "back": [
                {
                    "id": None,
                    "departurePoint": None,
                    "destinationPoint": None
                }
            ]
        })

        if not resultTo:
            raise HTTPException(status_code=404, detail="No valid paths found")
        return resultTo


@router.get("/negative")
async def get_class_names() -> List[str]:
    with open(".//neural_network/classification/class_names_negative.txt", "r", encoding="utf-8") as file:
        class_names = [line.strip() for line in file.readlines()]
    return class_names


@router.get("/positive")
async def get_class_names() -> List[str]:
    with open(".//neural_network/classification/class_names_positive.txt", "r", encoding="utf-8") as file:
        class_names = [line.strip() for line in file.readlines()]
    return class_names
